<?php
/*********************************************************
 * 日期：2021-01-07
 * 功能：云平台一键断网任务与一键断网消息推送。
 * 限制：只能后台命令执行
 *********************************************************
 */
if (PHP_SAPI != "cli") exit;
defined('ROOT_DIR') or define('ROOT_DIR', '/usr/logd/www/cloud');
defined('LOGDATA_DIR') or define('LOGDATA_DIR', '/usr/logdata');
define ('DIR_DEVOPS_COLLECTION', LOGDATA_DIR . '/devops/opssvc/collection');
define ('DIR_DEVOPS_REPORT', LOGDATA_DIR . '/devops/report');
define ("LOG_CONF", "/usr/logd/bin/log.conf");

define ('DEVOPS_DIR', LOGDATA_DIR . '/devops');
define ("CLOUD_CONF", "/etc/cloud.conf");
define ('DATAEYE', '/usr/logd/bin/cloudeye');
define ("ADMIN_ACCT", "admin");

require_once(ROOT_DIR . '/ni-core/class/ni_db.php');
require_once(ROOT_DIR . '/ni-core/comm.php');
require_once(ROOT_DIR . '/conf/db.php');


define('TIMEZONE_CONF', '/usr/logdata/timezone.conf');
$timezone_id = 'PRC';
if (file_exists(TIMEZONE_CONF) && $fp = fopen(TIMEZONE_CONF, "r")) {
	while (!feof($fp)) {
		if(($buf = fgets($fp)) === false)
			break;
		if(empty($buf = trim($buf, "\t\n\r\0\x0B")))
			continue;

		$col = explode('=', $buf, 2);
		if(count($col) != 2)
			continue;
		$key = trim($col[0]);
		$col[1] = trim($col[1]);
		if(strpos($key, ' ') !== false || empty($col[1]))
			continue;

		if($key != 'timezone_id')
			continue;

		$timezone_id = trim($col[1]);
		break;
	}
	fclose($fp);
}
date_default_timezone_set($timezone_id);

function gettokentask($serialno)
{
//	$key = strtolower($serialno);
	$cmd = DATAEYE . " device get devtoken={$serialno}";
	exec($cmd, $out, $ret);
	if($ret != 0)
		return array();

	$ret = array();
	foreach($out as $val) {
		$cols = explode('=', trim($val));
		if(count($cols) != 2)
			continue;
		$ret[$cols[0]] = $cols[1];
	}

	return $ret;
}

function is_supadmin($user)
{
	if($user == ADMIN_ACCT)
		return 1;

	$userfile = LOGDATA_DIR . "/supadmin/{$user}";
	if(file_exists($userfile)) {
		$ring = file_get_contents($userfile);
		$ring = json_decode($ring, true);
		if($ring) {
/*
			$ring = array(
				'forever' => 0,
				'expire' => 0,
			);
*/
			if(isset($ring['forever'])) {
				if($ring['forever'] == 1
				||($ring['forever'] == 2
				&& isset($ring['expire'])
				&& time() <= $ring['expire'])) {
					return 2;
				}
			}
		}
	}

	return 0;
}

function conslog($func, $err, $msg)
{
	if($err)
		echo "{$func}: [\033[38;5;1;4mfail\033[0m] {$msg}\n";
	else
		echo "{$func}: [\033[38;5;34;1msuccess\033[0m] {$msg}\n";
}

function _get_uuidgen()
{
	$ret = '';

	if (file_exists("/etc/uuidgen.conf")) {
		$ret = trim(file_get_contents("/etc/uuidgen.conf"), " \n\r\t\v\x00");
		if(empty($ret) == false)
			return $ret;
	}
	else {
		exec('/bin/uuidgen >> /etc/uuidgen.conf', $out, $ret);
		if($ret)
			conslog(__FUNCTION__, 1, "UUIDGEN.CONF_NEXISTS");
		else {
			$ret = trim(file_get_contents("/etc/uuidgen.conf"), " \n\r\t\v\x00");
			if(empty($ret) == false)
				return $ret;
		}
	}

	return false;
}

function _getconfig($tag, $def)
{
	$conf = LOG_CONF;

	if(!isset($def))
		$def = 0;
	$v = '';
	if (file_exists($conf)) {
		$lines = file($conf);
		foreach($lines as $val) {
			if (strstr($val, $tag) !== false) {
				$ds = explode('=', $val);
				$v = trim($ds[1], "\r\n");
				return $v;
			}
		}
	}
	
	return $def;
}

function _add_cloudconf($data)
{
	if (file_exists(CLOUD_CONF)) {
		$fp = fopen(CLOUD_CONF, 'r');
		if(!$fp) return false;
		
		$srcdata = array();
		while (!feof($fp)) {
			if(($buf = fgets($fp)) === false)
				break;

			$cols = explode('=', trim($buf, " \t\n\r\0\x0B\""), 2);
			if(($cnt = count($cols)) < 2)
				continue;
			$srcdata[$cols[0]] = $cols[1];
		}
		fclose($fp);
		
		foreach($data as $key => $val) 
			$srcdata[$key] = $val;

		$data = $srcdata;
	}

	if (($fp = fopen(CLOUD_CONF . '.cloud.web', "w")) === false)
		return false;

	foreach($data as $key => $val) 
		fwrite($fp, "{$key}={$val}\n");

	fclose($fp);

	if(file_exists(CLOUD_CONF))
		unlink(CLOUD_CONF);
	rename(CLOUD_CONF . '.cloud.web', CLOUD_CONF);

	return true;
}

function _gethttpshost()
{
	$result = array(
		'httpshost' => '',
		'httpsport'	=> '',
		'sshdport'	=> '',
	);

	if (file_exists(CLOUD_CONF)) {
		$lines = file(CLOUD_CONF);
		foreach($lines as $val) {
			if (strpos($val, "httpshost=") !== false) {
				$ds = explode('=', $val);
				$result['httpshost'] = trim($ds[1], "\n");
			}
			else
			if (strpos($val, "httpsport=") !== false) {
				$ds = explode('=', $val);
				$result['httpsport'] = intval(trim($ds[1], "\n"));
			}
			else
			if (strpos($val, "sshdport=") !== false) {
				$ds = explode('=', $val);
				$result['sshdport'] = intval(trim($ds[1], "\n"));
			}
		}
	}

	return $result;
}

function _getuser($openid)
{
	global $nidb;
	
	$sql = "select * from cloud_platform.weixin_user where `openid` = ?";

	try {
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $openid, \PDO::PARAM_STR);
		$sth->execute();
		$rows = $sth->fetchAll(\PDO::FETCH_ASSOC);
	}
	catch (\NiDBException $e) {
		conslog(__FUNCTION__, 1, "sql, DBException=" . $e->getMessage());
		return false;
	}

	if(count($rows) > 0)
		return $rows[0];

	return false;
}

function _getacct($account)
{
	global $nidb;
	
	$sql = "select * from cloud_users where `username` = ? limit 1";

	try {
		$sth = $nidb->prepare($sql);

		$index = 1;
		$sth->bindParam($index++, $account, \PDO::PARAM_STR);

		$sth->execute();

		$row = $sth->fetch(\PDO::FETCH_ASSOC);
	} catch (\PDOException $e) {
		conslog(__FUNCTION__, 1, "sql, DBException=" . $e->getMessage());
		return false;
	}

	if(!$row) return false;

	$user = array();
	foreach($row as $key => $val)
		$user[$key] = $val;

	return (object)$user;
}

function _findusers_from_grpid($grpid)
{
	global $nidb;

	try {
		$sql = "select d.`openid`, d.`user`, d.`nick` from palog.cloud_users as `s`";
		$sql.= " inner join cloud_platform.weixin_user as `d`";
		$sql.= " on s.username = d.user";
		$sql.= " where (find_in_set(?, s.`grp`) or s.`username` = '" . ADMIN_ACCT . "')";
		$sql.= " and d.`user` != d.`openid` and d.`enable` != 0";
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $grpid, \PDO::PARAM_STR);
		if($sth->execute()) {
			$rows = $sth->fetchAll(\PDO::FETCH_ASSOC);
			
			if(count($rows))
				return $rows;
		}
	}
	catch (\NiDBException $e) {
		conslog(__FUNCTION__, 1, "sql, DBException=" . $e->getMessage());
		return false;
	}

	return false;
}

function cloud_insertlog($user, $ip, $cont)
{
	global $nidb;

	$frmData = array(
			'user' => $user,
			'time' => time(),
			'cont' => $cont,
			'ipaddr' => $ip
		);
	try {
		if(insert_data('cloud_log', $frmData) === false) {
				$errmsg = implode(' ', $nidb->errorInfo());
				conslog(__FUNCTION__, 1, $errmsg);
				return false;
		}
		$id = $nidb->lastInsertId();
	}
	catch (\NiDBException $e) {
		conslog(__FUNCTION__, 1, "sql, DBException=" . $e->getMessage());
		return false;
	}

	return $id;
}

function getdev($grpids, $keyno)
{
	global $nidb;


	if(count($grpids) > 0) {
		$str = "?";
		if(count($grpids) > 1)
			$str.= str_repeat(',?', count($grpids) - 1);

		$str = "`grpid` in ({$str}) and ";
	}
	else
		$str = "";

	try {
		$sql = "select `license_id12`, `grpid`,`sys_name`,`sys_name2` from cloud_device where {$str}`license_id12` = ?";
		$sth = $nidb->prepare($sql);
		for($i = 0; $i < count($grpids); $i++) {
			$index = $i + 1;
			$sth->bindParam($index, $grpids[$i], \PDO::PARAM_INT);
		}
		$index = $i + 1;
		$sth->bindParam($index, $keyno, \PDO::PARAM_STR);
		if($sth->execute())
			$row = $sth->fetch(\PDO::FETCH_ASSOC);
	}
	catch (\NiDBException $e) {
		conslog(__FUNCTION__, 1, "sql, DBException=" . $e->getMessage());
		return false;
	}

	if(!$row) {
		conslog(__FUNCTION__, 1, "'{$keyno}' not foud device");
		return false;
	}

	if ($row['sys_name'] != "") {
		$sys_name = iconv("gbk", "utf-8//translit", $row['sys_name']);
		if($sys_name)
			$row['sys_name'] = $sys_name;
	}

	if ($row['sys_name2'] != "") {
		$sys_name = iconv("gbk", "utf-8//translit", $row['sys_name2']);
		if($sys_name)
			$row['sys_name2'] = $sys_name;
	}
	
	if($row['sys_name2'])
		$row['name'] = $row['sys_name2'];
	else
		$row['name'] = $row['sys_name'];
		
	return $row;
}

function addlog($data)
{
	global $nidb;


	$optional = array();
	if(isset($data['status']) == false) 
		return false;

	if(format_and_push($data, 'device', $optional, '', 'string', false) == false) {
		return false;
	}
	if(format_and_push($data, 'openid', $optional, '', 'string', false) == false) {
		return false;
	}
	if(format_and_push($data, 'object', $optional, '', 'string', false) == false) {
		return false;
	}
	if(format_and_push($data, 'username', $optional, '', 'string', false) == false) {
		return false;
	}

	if(format_and_push($data, 'type', $optional, '', 'int', false) == false)
		$optional['type'] = 2;

	format_and_push($data, 'avatarurl', $optional, '', 'string', false);
	format_and_push($data, 'nickname', $optional, '', 'string', false);
	format_and_push($data, 'ipaddr', $optional, '', 'string', false);
	
	if(isset($optional['avatarurl']))
		$optional['avatarurl'] = urldecode($optional['avatarurl']);
	if(isset($optional['nickname']))
		$optional['nickname'] = urldecode($optional['nickname']);

	$optional['status'] = $data['status'];
	$optional['birth'] = time();

	try {
		if(insert_data('weixin_brokennet_operation', $optional) === false) {
			return false;
		}
		$optional['id'] = $nidb->lastInsertId();
	}
	catch (\NiDBException $e) {
		return false;
	}

	return $optional;
}

function group_file_merge($dir)
{
	$fils = array();

	if (is_dir($dir) && ($dh = opendir($dir))) {
		while (($file = readdir($dh)) !== false) {
			if($file != "." && $file != "..") {
				if (is_file($dir . '/' . $file) && substr($file, -4) == '.lst') {
					array_push($fils, $file);
				}
			}
		}
		closedir($dh);
	}
	
	$file = "{$dir}.lst.tmp";
	$wfp = fopen($file, "w");
	if($wfp) {
		foreach($fils as $f) {
			$fp = fopen("{$dir}/{$f}", "r");
			if($fp) {
				while(!feof($fp) && ($line = fgets($fp, 2048)) !== false)
					fwrite($wfp, $line);

				fclose($fp);
			}
		}
		fclose($wfp);
		
		if(count($fils) > 0) {
			exec("sort {$dir}.lst.tmp | uniq > {$dir}.lst.uniq", $out, $ret);
			if($ret)
				unlink("{$dir}.lst.uniq");
			else {
				unlink($file);
				$file = "{$dir}.lst.uniq";
			}
		}

		if(file_exists("{$dir}.lst"))
			unlink("{$dir}.lst");

		rename($file, "{$dir}.lst");
	}
}

function update_task($task, $keyno, $hasip, $hasdomain, $count)
{
	if (file_exists(CLOUD_CONF)) {
		$lines = file(CLOUD_CONF);
		foreach($lines as $val) {
			if (strpos($val, "httpsport=") !== false) {
				$ds = explode('=', $val);
				$port = intval(trim($ds[1], "\n"));
				break;
			}
		}
	}

	if(isset($port))
		$port = ":{$port}";
	else
		$port = "";

	// $hasip = $hasdomain = 1;
	// 2021-11-18
	// 上海电信能力开放平台
	// 区域反诈应用北向接口协议规范
	// 需求，同步IP黑名单与域名黑名单至PA则：
	//	1、下发黑名单。
	//	2、合并本地黑名单，当列表为空时表示清空当前下发的所有资源
	//	3、导入资源：云平台禁止上网域名、云平台禁止上网IP
	//  4、生成 控制策略

	$shell = '#!/bin/sh
[ ! -f /etc/PG.conf ] && echo "NO_PG_CONF" && exit 255
. /etc/PG.conf
[ "${PGETC}" = "" ] && echo "NO_PGETC" && exit 255
[ "${DATAPATH}" = "" ] && echo "NO_DATAPATH" && exit 255
[ ! -f ${PGETC}/App/cloud/cloud.conf ] && echo "NO_CLOUD_CONF" && exit 255
MT_IPADDR=`grep ^MT_IPADDR= ${PGETC}/App/cloud/cloud.conf | cut -d\'=\' -f2`
[ "${MT_IPADDR}" = "" ] && echo "NO_MT_IPADDR" && exit 255

MYROOT=/usr/ramdisk/app/cloud
TASK_MAIN=${MYROOT}/bin/task_main
TASK_CLIENT=${MYROOT}/bin/task_client
[ ! -f ${TASK_MAIN} ] && echo "NOFOUND_TASK_MAIN" && exit 255
[ ! -f ${TASK_CLIENT} ] && echo "NOFOUND_TASK_CLIENT" && exit 255
PORT=' . $port . '
';
if($hasip) {
	// 合并IP资源
	$dir = DEVOPS_DIR . "/web/policy/gateway/{$keyno}/ip/netoff";
	group_file_merge($dir);
	$ipgrpname = iconv('UTF-8', 'gb2312', '云平台禁止上网IP');

	// allow= deny=
	$status = "{$count['allowip']} {$count['denyip']}";
	
	$shell.= '
url="https://${MT_IPADDR}${PORT}/devops/policy/gateway/' . $keyno . '/ip/netoff.lst"
wfile=${DATAPATH}/App/cloud/policy/ip/' . $ipgrpname . '.lst
[ -f ${wfile} ] && echo -n "

" > ${wfile}
touch ${wfile}
${TASK_MAIN} ${url} ${wfile}
${TASK_CLIENT} policy_mk_ip ' . $ipgrpname . '
if [ $? -eq 0 ]; then
	echo "RET IP ' . "{$keyno} {$status}" . '"
fi
';
}

if($hasdomain) {
	// 合并域名资源 
	$dir = DEVOPS_DIR . "/web/policy/gateway/{$keyno}/domain/netoff";
	group_file_merge($dir);
	$domaingrpname = iconv('UTF-8', 'gb2312', '云平台禁止上网域名');

	// allow= deny=
	$status = "{$count['allowdomain']} {$count['denydomain']}";
	
	$shell.= '
url="https://${MT_IPADDR}${PORT}/devops/policy/gateway/' . $keyno . '/domain/netoff.lst"
wfile=${DATAPATH}/App/cloud/policy/domain/' . $domaingrpname . '.lst
[ -f ${wfile} ] && rm -f ${wfile}
touch ${wfile}
${TASK_MAIN} ${url} ${wfile}
${TASK_CLIENT} policy_mk_domain ' . $domaingrpname . '
if [ $? -eq 0 ]; then
	echo "RET DOMAIN ' . "{$keyno} {$status}" . '"
fi
';
}

	$fbinsh = "{$task['taskpath']}/{$task['token']}/bin/blackip_sync";
	file_put_contents("{$fbinsh}.tmp", $shell);
	if(file_exists($fbinsh))
		unlink($fbinsh);
	rename("{$fbinsh}.tmp", $fbinsh);

	$fret = "{$task['taskpath']}/{$task['token']}/result/blackip_sync";
	if(file_exists($fret))
		unlink($fret);

	$forder = "{$task['taskpath']}/{$task['token']}/order/blackip_sync";
	return file_put_contents($forder, "");
}

function dumptofile($keyno) 
{
	global $nidb;

	$task = gettokentask($keyno);
	if(isset($task['token']) == false || $task['token'] == '')
		return '';

	try {
		$sql = "select `type` from weixin_brokennet";
		$sql.= " where `license_id12` = ? and `isupdate` != 0 group by `type`";

		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $keyno, \PDO::PARAM_STR);
		if(!$sth->execute()) 
			return '';

		$types = $sth->fetchAll(\PDO::FETCH_COLUMN, 0);
		if(count($types) == 0) {
			$count = array(
				'allowip' => 0,
				'denyip' => '清空',
				'allowdomain' => 0,
				'denydomain' => '清空',
			);
			$ipdir = DEVOPS_DIR . "/web/policy/gateway/{$keyno}/ip/netoff";
			if(file_exists("{$ipdir}/cloud.lst"))
				unlink("{$ipdir}/cloud.lst");
		
			$domaindir = DEVOPS_DIR . "/web/policy/gateway/{$keyno}/domain/netoff";
			if(file_exists("{$domaindir}/cloud.lst"))
				unlink("{$domaindir}/cloud.lst");

			update_task($task, $keyno, true, true, $count);
			return '';
		}

		$sql = "select `myevent`, `type`, `isupdate`, `status` from weixin_brokennet";
		$sql.= " where `license_id12` = ? and `type` in (" . implode(',', $types) . ")";

		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $keyno, \PDO::PARAM_STR);
		if($sth->execute()) {
			$hasip = in_array(0, $types);
			$hasdomain = in_array(1, $types);
			
			if($hasip) {
				$ipdir = DEVOPS_DIR . "/web/policy/gateway/{$keyno}/ip/netoff";
				if(!is_dir($ipdir))
					exec("mkdir -p {$ipdir}");

				$ipfp = fopen("{$ipdir}/cloud.lst.tmp", "w");
				if(!$ipfp) {
					conslog(__FUNCTION__, 1, "fopen={$ipdir}/cloud.lst");
					return '';
				}
			}
			
			if($hasdomain) {
				$domaindir = DEVOPS_DIR . "/web/policy/gateway/{$keyno}/domain/netoff";
				if(!is_dir($domaindir))
					exec("mkdir -p {$domaindir}");

				$domainfp = fopen("{$domaindir}/cloud.lst.tmp", "w");
				if(!$domainfp) {
					if($hasip) fclose($ipfp);
					conslog(__FUNCTION__, 1, "fopen={$domaindir}/cloud.lst");
					return '';
				}
			}
			
			$count = array(
				'allowip' => 0,
				'denyip' => 0,
				'allowdomain' => 0,
				'denydomain' => 0,
			);

			while ($row = $sth->fetch(\PDO::FETCH_ASSOC, \PDO::FETCH_ORI_NEXT)) {
				if($row['type']) {
					if($row['status'] == 0) {
						if($row['isupdate']) {
							$count['denydomain']++;
							$denydomain = $row['myevent'];
						}
						fwrite($domainfp, "{$row['myevent']}\n");
					}
					else
					if($row['isupdate']) {
						$count['allowdomain']++;
						$allowdomain = $row['myevent'];
					}
				}
				else {
					if($row['status'] == 0) {
						if($row['isupdate']) {
							$count['denyip']++;
							$denyip = $row['myevent'];
						}
						fwrite($ipfp, "{$row['myevent']}\n");
					}
					else
					if($row['isupdate']) {
						$count['allowip']++;
						$allowip = $row['myevent'];
					}
				}
			}
			if($hasip) {
				fclose($ipfp);
				if(file_exists("{$ipdir}/cloud.lst"))
					unlink("{$ipdir}/cloud.lst");
				rename("{$ipdir}/cloud.lst.tmp", "{$ipdir}/cloud.lst");
			}
			if($hasdomain) {
				fclose($domainfp);
				if(file_exists("{$domaindir}/cloud.lst"))
					unlink("{$domaindir}/cloud.lst");
				rename("{$domaindir}/cloud.lst.tmp", "{$domaindir}/cloud.lst");
			}
			
			if($count['allowip'] == 1)
				$count['allowip'] = $allowip;
			if($count['denyip'] == 1)
				$count['denyip'] = $denyip;
			if($count['allowdomain'] == 1)
				$count['allowdomain'] = $allowdomain;
			if($count['denydomain'] == 1)
				$count['denydomain'] = $denydomain;

			update_task($task, $keyno, $hasip, $hasdomain, $count);

			return implode(',', $types);
		}
	}
	catch (\NiDBException $e) {
		conslog(__FUNCTION__, 1, "sql, DBException=" . $e->getMessage());
		return '';
	}

	return '';
}

function checkresult($keyno) 
{
	global $nidb;

	$task = gettokentask($keyno);
	if(isset($task['token']) == false || $task['token'] == '')
		return '';

	$fret = "{$task['taskpath']}/{$task['token']}/result/blackip_sync";
	if(!file_exists($fret))
		return '';

	$fp = fopen($fret, 'r');
	if(!$fp)
		return '';

	$msg = array();
	while (!feof($fp)) {
		if(($buf = fgets($fp)) === false)
			break;

		$cols = explode(' ', trim($buf, " \t\n\r\0\x0B\""));
		if(count($cols) != 5)
			continue;

		if($cols[0] == 'RET' && $cols[2] == $keyno) {
			if($cols[1] == 'IP') {
				if($cols[4])
					array_push($msg, "阻断IP={$cols[4]}");
				if($cols[3])
					array_push($msg, "放行IP={$cols[3]}");
			}
			else
			if($cols[1] == 'DOMAIN'){
				if($cols[4])
					array_push($msg, "阻断域名={$cols[4]}");
				if($cols[3])
					array_push($msg, "放行域名={$cols[3]}");
			}
		}
	}
	fclose($fp);

	if(count($msg)) return implode('，', $msg);
	return '';
}

function setobj($now, $wxuser, $dev, $netoff, $tag, $tips, $ipstr, $domainstr, $userip, $srctype)
{
	global $nidb;

	$ipstr = trim(preg_replace(array('/ +/', '/,+/'), array('', ','), $ipstr), ' ,');
	$domainstr = trim(preg_replace(array('/ +/', '/,+/'), array('', ','), $domainstr), ' ,');
	if($ipstr == '' && $domainstr == '')
		return;

	$ips = explode(',', $ipstr);
	$domains = explode(',', $domainstr);

	$logger = array(
			'type'		=> $srctype,
			'avatarurl' => $wxuser['img'],
			'device'	=> $dev['license_id12'],
			'devname'	=> $dev['name'],
			'openid'	=> $wxuser['openid'],
			'object'	=> '',
			'username'	=> $wxuser['user'],
			'nickname'	=> $wxuser['nick'],
			'status'	=> $netoff,
			'birth'		=> $now,
			'ipaddr'	=> $userip,
		);

	try {
		
		$update_sql = "update weixin_brokennet set";
		$update_sql.= " `sysname` = ?, `isupdate` = ?, `status` = ?, `tag` = ?, `tips` = ?";
		$update_sql.= " where `id` = ?";

		$fields = "`myevent`, `license_id12`, `sysname`, `isupdate`, `status`, `old_status`";
		$fields.= ", `tag`, `tips`, `type`, `birth`";
		$placeholder = "?,?,?,1,?,?,?,?,?,?";
		$insert_sql = "insert into `weixin_brokennet` ($fields) values ($placeholder)";

		$select_sql = "select `id`, `isupdate`, `old_status`, `sysname` from weixin_brokennet where `myevent` = ? and `license_id12` = ? and `type` = ?";
		
		// 添加 一键断网 IP对象
		$type = 0;
		foreach ($ips as $ip) {
			if($ip == '')
				continue;

			$sth = $nidb->prepare($select_sql);
			$sth->bindParam(1, $ip, \PDO::PARAM_STR);
			$sth->bindParam(2, $dev['license_id12'], \PDO::PARAM_STR);
			$sth->bindParam(3, $type, \PDO::PARAM_INT);
			if(!$sth->execute()) {
				conslog(__FUNCTION__, 1, "keyno={$dev['license_id12']} netoff={$netoff} ip={$ip}");
				continue;
			}
			
			$row = $sth->fetch(\PDO::FETCH_ASSOC);
			if($row) {

				if($row['old_status'] != $netoff)
					$isupdate = 1;
				else
					$isupdate = 0;

				$sth = $nidb->prepare($update_sql);
				$sth->bindParam(1, $dev['name'], \PDO::PARAM_STR);
				$sth->bindParam(2, $isupdate, \PDO::PARAM_STR);
				$sth->bindParam(3, $netoff, \PDO::PARAM_INT);
				$sth->bindParam(4, $tag, \PDO::PARAM_STR);
				$sth->bindParam(5, $tips, \PDO::PARAM_STR);
				$sth->bindParam(6, $row['id'], \PDO::PARAM_INT);
				if(!$sth->execute()) {
					conslog(__FUNCTION__, 1, "keyno={$dev['license_id12']} netoff={$netoff} ip={$ip}");
					continue;
				}
			}
			else {
				$old_status = ($netoff?0:1);
				$sth = $nidb->prepare($insert_sql);
				$sth->bindParam(1, $ip, \PDO::PARAM_STR);
				$sth->bindParam(2, $dev['license_id12'], \PDO::PARAM_STR);
				$sth->bindParam(3, $dev['name'], \PDO::PARAM_STR);
				$sth->bindParam(4, $netoff, \PDO::PARAM_INT);
				$sth->bindParam(5, $old_status, \PDO::PARAM_INT);
				$sth->bindParam(6, $tag, \PDO::PARAM_STR);
				$sth->bindParam(7, $tips, \PDO::PARAM_STR);
				$sth->bindParam(8, $type, \PDO::PARAM_INT);
				$sth->bindParam(9, $now, \PDO::PARAM_INT);

				if(!$sth->execute()) {
					conslog(__FUNCTION__, 1, "keyno={$dev['license_id12']} netoff={$netoff} ip={$ip}");
					continue;
				}
			}

			$logger['object'] = $ip;
			addlog($logger);
		}

		// 添加 一键断网 域名对象
		$type = 1;
		foreach ($domains as $domain) {
			if($domain == '')
				continue;

			$sth = $nidb->prepare($select_sql);
			$sth->bindParam(1, $domain, \PDO::PARAM_STR);
			$sth->bindParam(2, $dev['license_id12'], \PDO::PARAM_STR);
			$sth->bindParam(3, $type, \PDO::PARAM_INT);
			if(!$sth->execute()) {
				conslog(__FUNCTION__, 1, "keyno={$dev['license_id12']} netoff={$netoff} ip={$ip}");
				continue;
			}

			$row = $sth->fetch(\PDO::FETCH_ASSOC);
			if($row) {

				if($row['old_status'] != $netoff)
					$isupdate = 1;
				else
					$isupdate = 0;

				$sth = $nidb->prepare($update_sql);
				$sth->bindParam(1, $dev['name'], \PDO::PARAM_STR);
				$sth->bindParam(2, $isupdate, \PDO::PARAM_STR);
				$sth->bindParam(3, $netoff, \PDO::PARAM_INT);
				$sth->bindParam(4, $tag, \PDO::PARAM_STR);
				$sth->bindParam(5, $tips, \PDO::PARAM_STR);
				$sth->bindParam(6, $row['id'], \PDO::PARAM_INT);
				if(!$sth->execute()) {
					conslog(__FUNCTION__, 1, "keyno={$dev['license_id12']} netoff={$netoff} ip={$ip}");
					continue;
				}
			}
			else {
				$old_status = ($netoff?0:1);
				$sth = $nidb->prepare($insert_sql);
				$sth->bindParam(1, $domain, \PDO::PARAM_STR);
				$sth->bindParam(2, $dev['license_id12'], \PDO::PARAM_STR);
				$sth->bindParam(3, $dev['name'], \PDO::PARAM_STR);
				$sth->bindParam(4, $netoff, \PDO::PARAM_INT);
				$sth->bindParam(5, $old_status, \PDO::PARAM_INT);
				$sth->bindParam(6, $tag, \PDO::PARAM_STR);
				$sth->bindParam(7, $tips, \PDO::PARAM_STR);
				$sth->bindParam(8, $type, \PDO::PARAM_INT);
				$sth->bindParam(9, $now, \PDO::PARAM_INT);

				if(!$sth->execute()) {
					conslog(__FUNCTION__, 1, "keyno={$dev['license_id12']} netoff={$netoff} ip={$ip}");
					continue;
				}
			}

			$logger['object'] = $domain;
			addlog($logger);
		}

	}
	catch (\NiDBException $e) {
		conslog(__FUNCTION__, 1, "sql, DBException=" . $e->getMessage());
		return false;
	}

	return true;
}

function analysis_cmd($data)
{
/*
	$data = json_decode($cmd, true);
	if(json_last_error() != JSON_ERROR_NONE) {
		conslog(__FUNCTION__, 1, "json_decode, data={$cmd}");
		return false;
	}
*/
	if(!isset($data['rows']) 
	|| !is_array($data['rows'])) {
		conslog(__FUNCTION__, 1, "invdata, ret=" . json_encode($data));
		return false;
	}

	if(!isset($data['openid'])
	|| $data['openid'] == "") {
		conslog(__FUNCTION__, 1, "invdata, openid empty");
		return false;
	}

	// 检查会话来源类型
	if(!isset($data['type'])
	|| ($data['type'] = intval($data['type'])) == 0) {
		conslog(__FUNCTION__, 1, "invdata, type error.");
		return false;
	}
	
	// 操作者的IP地址
	if(!isset($data['_USERIP_'])
	|| ($USERIP = trim($data['_USERIP_'])) == "")
		$USERIP = $host;

	if(!($wxuser = _getuser($data['openid']))) {
		conslog(__FUNCTION__, 1, "invalid openid {$data['openid']}");
		return false;
	}

	if(!$wxuser['enable']) {
		conslog(__FUNCTION__, 1, "user disabled");
		return false;
	}

	if(!($user = _getacct($wxuser['user']))) {
		conslog(__FUNCTION__, 1, "invalid user");
		return false;
	}

	if (!is_supadmin($user->username)) {
		if($user->grp == '') {
			conslog(__FUNCTION__, 1, "invalid ring of '{$user->username}'");
			return false;
		}

		$grpids = explode(',', trim(preg_replace(array('/ +/', '/,+/'), array('', ','), $user->grp), ' ,'));
	}
	else
		$grpids = array();

	$now = time();
	$devs = array();
	foreach($data['rows'] as $line) {
		$cols = explode(' ', $line);
		$val = array();
		foreach($cols as $col) {
			$cols = explode('=', $col, 2);
			if(count($cols) == 2)
				$val[$cols[0]] = $cols[1];
		}

		if(!isset($val['device'])
		|| empty($val['device'])) 
			continue;

		if(isset($val['deny']) && isset($val[$val['deny']]) && !empty($val[$val['deny']])) {
			// openid= device= deny=host host=www.test.com
			if(isset($devs[$val['device']]))
				$dev = $devs[$val['device']];
			else {
				$dev = getdev($grpids, $val['device']);
				if(!$dev) 
					continue;
				$devs[$val['device']] = $dev;
			}
			
			if(isset($val['tag']))
				$tag = $val['tag'];
			else
				$tag = '';

			if(isset($val['remark']))
				$remark = $val['remark'];
			else
				$remark = '';

			if($val['deny'] == 'host')
				setobj($now, $wxuser, $dev, 1, $tag, $remark, '', $val[$val['deny']], $USERIP, $data['type']);
			else
				setobj($now, $wxuser, $dev, 1, $tag, $remark, $val[$val['deny']], '', $USERIP, $data['type']);

			continue;
		}

		if(isset($val['allow']) && isset($val[$val['allow']]) && !empty($val[$val['allow']])) {
			// openid= device= deny=ip ip=192.168.11.23
			if(isset($devs[$val['device']]))
				$dev = $devs[$val['device']];
			else {
				$dev = getdev($grpids, $val['device']);
				if(!$dev) 
					continue;
				$devs[$val['device']] = $dev;
			}

			if(isset($val['tag']))
				$tag = $val['tag'];
			else
				$tag = '';

			if(isset($val['remark']))
				$remark = $val['remark'];
			else
				$remark = '';

			if($val['allow'] == 'host')
				setobj($now, $wxuser, $dev, 0, $tag, $remark, '', $val[$val['allow']], $USERIP, $data['type']);
			else
				setobj($now, $wxuser, $dev, 0, $tag, $remark, $val[$val['allow']], '', $USERIP, $data['type']);

			continue;
		}

		if(isset($val['netoff'])) {
			if(isset($val['ip']) && !empty($val['ip'])) {
				$ip = $val['ip'];
				// openid= device= netoff=allow|deny/0|1 ip=192.168.11.23,192.168.11.24,...
			}
			else
				$ip = '';

			if(isset($val['domain']) && !empty($val['domain'])) {
				$domain = $val['domain'];
				// openid= device= netoff=allow|deny/0|1 domain=www.test.com,www.ok.com,...
			}
			else
				$domain = '';

			if($ip == '' && $domain == '')
				continue;

			if(isset($devs[$val['device']]))
				$dev = $devs[$val['device']];
			else {
				$dev = getdev($grpids, $val['device']);
				if(!$dev) 
					continue;
				$devs[$val['device']] = $dev;
			}

			if(isset($val['tag']))
				$tag = $val['tag'];
			else
				$tag = '';

			if(isset($val['remark']))
				$remark = $val['remark'];
			else
				$remark = '';

			if('allow' == $val['netoff'] || intval($val['netoff']) == 0)
				setobj($now, $wxuser, $dev, 0, $tag, $remark, $ip, $domain, $USERIP, $data['type']);
			else
				setobj($now, $wxuser, $dev, 1, $tag, $remark, $ip, $domain, $USERIP, $data['type']);
		}
	}
	
	return true;
}

function fetchcmd($cid)
{
	$host = "weixin.panabit.com";
	$url = "https://{$host}/panacloud/fetchcmd.php";
	$ret = httpPost($url, array('cid' => $cid), true);
	if(empty($ret)) {
		conslog(__FUNCTION__, 1, "httpPost({$url}) empty.");
		return false;
	}

	$data = json_decode($ret, true);
	if(json_last_error() != JSON_ERROR_NONE) {
		conslog(__FUNCTION__, 1, "json_decode, data={$ret}");
		return false;
	}

	return analysis_cmd($data);
}

function cmd_netoff_result()
{
	global $nidb;


	$devs = array();
	// 检查上次的任务同步结果
	try {
		$sql = "select `id`, `license_id12` from weixin_brokennet";
		$sql.= " where `isupdate` = 2 and `license_id12` != '' and `myevent` != ''";
		$sql.= " group by `license_id12` limit 10";

		$sth = $nidb->prepare($sql);
		if($sth->execute()) {
			$rows = $sth->fetchAll(\PDO::FETCH_ASSOC);

			$path = LOGDATA_DIR . "/weixin";
			if(!is_dir($path))
				exec("mkdir -p {$path}");
			
			foreach($rows as $row) {
				if(($msg = checkresult($row['license_id12'])) != '') {
					$sql = "update weixin_brokennet set `isupdate` = 0, `old_status` = `status` where `isupdate` = 2 and `license_id12` = ?"; // and `type` in ({$types})";
					$sth = $nidb->prepare($sql);
					$sth->bindParam(1, $row['license_id12'], \PDO::PARAM_STR);
					$sth->execute();
					conslog(__FUNCTION__, 0, "complete keyno={$row['license_id12']} msg={$msg}");

					file_put_contents("{$path}/{$row['license_id12']}.msg", $msg);
				}
			}
		}
	}
	catch (\NiDBException $e) {
		conslog(__FUNCTION__, 1, "sql, DBException=" . $e->getMessage());
		return false;
	}
}

function cmd_netoff_send()
{
	global $nidb;


	$delkeyno = array();
	$file = LOGDATA_DIR . '/weixin/delete_keyno.lst';
	if(file_exists($file)) {
		if(file_exists("{$file}.read") == false)
			rename($file, "{$file}.read");
		$fp = fopen("{$file}.read", 'r');
		if($fp) {
			while (!feof($fp)) {
				if(($buf = fgets($fp)) === false)
					break;

				$keyno = trim($buf, " \t\n\r\0\x0B\"");
				if($keyno == '')
					continue;

				if(in_array($keyno, $delkeyno) == false) 
					array_push($delkeyno, $keyno);
			}
			fclose($fp);
			unlink("{$file}.read");
		}

		foreach($delkeyno as $keyno) {
			if(($types = dumptofile($keyno)) != '') {
				conslog(__FUNCTION__, 0, "sending keyno={$keyno} type={$types}");
			}
		}
	}
	
	// 生成同步任务
	try {
		$sql = "select `id`, `license_id12` from weixin_brokennet";
		$sql.= " where `isupdate` = 1 and `license_id12` != '' and `myevent` != ''";
		$sql.= " group by `license_id12` limit 10";

		$sth = $nidb->prepare($sql);
		if($sth->execute()) {
			$rows = $sth->fetchAll(\PDO::FETCH_ASSOC);
			
			$dumpok = array();
			foreach($rows as $row) {
				if(in_array($row['license_id12'], $delkeyno)) {
					array_push($dumpok, $row['license_id12']);
					continue;
				}

				if(($types = dumptofile($row['license_id12'])) != '') {
					array_push($dumpok, $row['license_id12']);
					conslog(__FUNCTION__, 0, "sending keyno={$row['license_id12']} type={$types}");
				}
			}

			if(count($dumpok)) {
				$sql = "update weixin_brokennet set `isupdate` = 2 where `isupdate` = 1 and `myevent` != '' and `license_id12` in (?" . str_repeat(',?', count($dumpok) - 1) . ")";
				$sth = $nidb->prepare($sql);
				$sth->execute($dumpok);
			}
		}
	}
	catch (\NiDBException $e) {
		conslog(__FUNCTION__, 1, "sql, DBException=" . $e->getMessage());
		return false;
	}

	return true;
}

function sendmsg_netoff($cid)
{
	global $nidb;

	// 扫描消息文件
	$timeout = time() - 300;
	$path = LOGDATA_DIR . "/weixin";
	$files = array();
	if (($dh = opendir($path))) {
		while (($file = readdir($dh)) !== false) {
			if($file != "." && $file != "..") {
				if (substr($file, -4) == '.msg') {
					$fname = $path . '/' . $file;
					if(is_file($fname)) {
						$ctm = filectime($fname);
						if($ctm > $timeout) {
							array_push($files, array(
								'keyno' => substr($file, 0, 12),
								'birth'	=> filectime($fname),
								'msg' => file_get_contents($fname)
							));
						}
						else
							unlink($fname);
					}
				}
			}
		}
		closedir($dh);
	}

	$url = "https://weixin.panabit.com/panacloud/sendmsg.php";
	foreach($files as $row) {
		$dev = getdev(array(), $row['keyno']);
		if(!$dev) {
			unlink("{$path}/{$row['keyno']}.msg");
			continue;
		}
		$users = _findusers_from_grpid($dev['grpid']);
		if($users === false) {
			unlink("{$path}/{$row['keyno']}.msg");
			continue;
		}

		$row['type']= 'netoff';
		$row['cid'] = $cid;
		foreach($users as $user) {
			$row['name'] = $dev['name'];
			$row['acct'] = $user['user'];
			$row['nick'] = $user['nick'];
			$row['openid']= $user['openid'];
			$ret = httpPost($url, $row, true);
			if(empty($ret)) {
				conslog(__FUNCTION__, 1, "httpPost({$url}) empty.");
				return false;
			}

			$errmsg = "keyno={$row['keyno']} birth=". date('Y-m-d H:i:s', $row['birth']) . " msg={$row['msg']}";
			
			$data = json_decode($ret, true);
			if(json_last_error() != JSON_ERROR_NONE) {
				conslog(__FUNCTION__, 1, "json_decode, {$errmsg} data={$ret}");
				continue;
			}

			if(!isset($data['ret'])
			|| !isset($data['data'])) {
				conslog(__FUNCTION__, 1, "invdata, {$errmsg} data={$ret}");
				continue;
			}

			if($data['ret']) {
				conslog(__FUNCTION__, 1, "{$errmsg} ret={$data['ret']} msg={$data['msg']}");
				continue;
			}
			
			conslog(__FUNCTION__, 0, "{$errmsg}");
			unlink("{$path}/{$row['keyno']}.msg");
		}
	}
}

function rebind($cid)
{
	global $nidb;

	$now = time();
	$sql = "select `openid`, `user`, `nick`, `expire_time` from cloud_platform.weixin_user where `expire_time` > {$now} limit 20";

	try {
		$sth = $nidb->prepare($sql);
		$sth->execute();
		$rows = $sth->fetchAll(\PDO::FETCH_ASSOC);
	}
	catch (\NiDBException $e) {
		conslog(__FUNCTION__, 1, "DBException=" . $e->getMessage());
		return false;
	}

	if(!$rows || count($rows) <= 0)
		return false;

	$sqlfind = "select `user`, `enable` from cloud_platform.weixin_user where `openid` = ?";
	$sqlrmv = "update cloud_platform.weixin_user set `enable` = 0 where `openid` = ?";
	$sqldelete = "delete from cloud_platform.weixin_user where `openid` = ?";
	$sqlreset = "update cloud_platform.weixin_user set `expire_time` = 0 where `openid` = ?";
	
	$sqlupdate = "update cloud_platform.weixin_user";
	$sqlupdate.= " set `bind_time` = ?, `nick` = ? , `img` = ?, `openid` = ?, `unionid` = ?, `expire_time` = 0, `enable` = 1";
	$sqlupdate.= " where `openid` = ? and `user` = ? and `expire_time` = ?";

	$arg = array(
		'cid'	=> $cid,
		'code'	=> '',
		'acct'	=> ''
	);
	$delete_openids = array();
	$url = "https://weixin.panabit.com/panacloud/rebind.php";
	foreach($rows as $row) {
		if(in_array($row['openid'], $delete_openids)) continue;
		$arg['code'] = $row['expire_time'];
		$arg['acct'] = $row['user'];
		$ret = httpPost($url, $arg, true);
		if(empty($ret)) {
			conslog(__FUNCTION__, 1, "httpPost({$url}) empty.");
			return false;
		}

		$data = json_decode($ret, true);
		if(json_last_error() != JSON_ERROR_NONE) {
			conslog(__FUNCTION__, 1, "json_decode, user={$row['user']} code={$row['expire_time']} data={$ret}");
			continue;
		}

		if(!isset($data['ret'])
		|| !isset($data['data'])) {
			conslog(__FUNCTION__, 1, "invdata, user={$row['user']} code={$row['expire_time']} data={$ret}");
			continue;
		}

		if($data['ret']) {
			conslog(__FUNCTION__, 1, "user={$row['user']} code={$row['expire_time']} openid={$row['openid']} ret={$data['ret']} msg={$data['msg']}");
			continue;
		}

		if(!isset($data['data']['openid'])
		|| empty($data['data']['openid'])) {
			conslog(__FUNCTION__, 1, "openid is empty, user={$row['user']} code={$row['expire_time']} openid={$row['openid']} data={$ret}");
			continue;
		}

		try {
			$sth = $nidb->prepare($sqlfind);
			$sth->bindParam(1, $data['data']['openid'], \PDO::PARAM_STR);
			$sth->execute();
			$userinfo = $sth->fetch(\PDO::FETCH_ASSOC);

			if($userinfo) {
				// 已存在，且是新增的则删除新增配置项。
				if($row['openid'] == $row['user']) {
					$sth = $nidb->prepare($sqlrmv);
					$sth->bindParam(1, $row['openid'], \PDO::PARAM_STR);
					if($sth->execute())
						conslog(__FUNCTION__, 1, "openid exists, user={$row['user']} code={$row['expire_time']} openid={$data['data']['openid']}");
					else
						conslog(__FUNCTION__, 1, "user={$row['user']} code={$row['expire_time']} openid={$row['openid']} DBException=" . $e->getMessage());
				}
				else
				if($row['openid'] != $data['data']['openid']
				&& $row['user'] == $userinfo['user']) {
					conslog(__FUNCTION__, 1, "openid exists, user={$row['user']} code={$row['expire_time']} openid={$data['data']['openid']}");

					$sth = $nidb->prepare($sqlreset);
					$sth->bindParam(1, $now, \PDO::PARAM_STR);
					$sth->bindParam(2, $row['openid'], \PDO::PARAM_STR);
					$sth->execute();
					continue;
				}

				// 删除原OPENID关联
				if($row['openid'] != $data['data']['openid']) {
					$sth = $nidb->prepare($sqldelete);
					$sth->bindParam(1, $data['data']['openid'], \PDO::PARAM_STR);
					$sth->execute();
					array_push($delete_openids, $data['data']['openid']);
				}
			}

			// 更新
			$sth = $nidb->prepare($sqlupdate);
			$sth->bindParam(1, $now, \PDO::PARAM_INT);
			$sth->bindParam(2, $data['data']['nickname'], \PDO::PARAM_STR);
			$sth->bindParam(3, $data['data']['headimgurl'], \PDO::PARAM_STR);
			$sth->bindParam(4, $data['data']['openid'], \PDO::PARAM_STR);
			$sth->bindParam(5, $data['data']['unionid'], \PDO::PARAM_STR);
			$sth->bindParam(6, $row['openid'], \PDO::PARAM_STR);
			$sth->bindParam(7, $row['user'], \PDO::PARAM_STR);
			$sth->bindParam(8, $row['expire_time'], \PDO::PARAM_INT);
			if($sth->execute()) {
				conslog(__FUNCTION__, 0, "user={$row['user']} code={$row['expire_time']} openid={$data['data']['openid']}");
				cloud_insertlog($row['user'], $data['data']['clientip'], "通过扫码关联微信：昵称={$data['data']['nickname']}，OPENID={$data['data']['openid']}");
			}
			else
				conslog(__FUNCTION__, 1, "user={$row['user']} code={$row['expire_time']} openid={$row['openid']} err=" . $e->getMessage());
		}
		catch (\NiDBException $e) {
			conslog(__FUNCTION__, 1, "user={$row['user']} code={$row['expire_time']} openid={$row['openid']} DBException=" . $e->getMessage());
			return false;
		}
	}

	return true;
}

function unbind($cid)
{
	global $nidb;

	$sql = "select `openid`, `user` from cloud_platform.weixin_user where `enable` = 0 and `user` != `openid` limit 20";

	try {
		$sth = $nidb->prepare($sql);
		$sth->execute();
		$rows = $sth->fetchAll(\PDO::FETCH_ASSOC);
	}
	catch (\NiDBException $e) {
		conslog(__FUNCTION__, 1, "DBException=" . $e->getMessage());
		return false;
	}

	if(!$rows || count($rows) <= 0)
		return false;

	$sqldelete = "delete from cloud_platform.weixin_user where `openid` = ?";

	$arg = array(
		'cid'	=> $cid,
		'openid'=> '',
		'acct'	=> ''
	);
	$delete_openids = array();
	$url = "https://weixin.panabit.com/panacloud/unbind.php";
	foreach($rows as $row) {
		if(in_array($row['openid'], $delete_openids)) continue;
		$arg['openid'] = $row['openid'];
		$arg['acct'] = $row['user'];
		$ret = httpPost($url, $arg, true);
		if(empty($ret)) {
			conslog(__FUNCTION__, 1, "httpPost({$url}) empty.");
			return false;
		}
		
		$data = json_decode($ret, true);
		if(json_last_error() != JSON_ERROR_NONE) {
			conslog(__FUNCTION__, 1, "json_decode, user={$row['user']} data={$ret}");
		}

		if($data['ret']) {
			conslog(__FUNCTION__, 1, "user={$row['user']} ret={$data['ret']} msg={$data['msg']}");
		}

		try {
			$sth = $nidb->prepare($sqldelete);
			$sth->bindParam(1, $row['openid'], \PDO::PARAM_STR);
			$sth->execute();
			if($sth->execute()) 
				conslog(__FUNCTION__, 0, "user={$row['user']} openid={$row['openid']}");
			else
				conslog(__FUNCTION__, 1, "user={$row['user']} openid={$row['openid']} err=" . $e->getMessage());
		}
		catch (\NiDBException $e) {
			conslog(__FUNCTION__, 1, "user={$row['user']} openid={$row['openid']} DBException=" . $e->getMessage());
			return false;
		}
	}

	return true;
}

function checkguid($cid)
{
	global $nidb;
	
	$conf = _gethttpshost();
	$wxname = trim(_getconfig("cloud_weixin_name", ''));

	$url = "https://weixin.panabit.com/panacloud/guid.php";
	$data = array(
		'cid'  => $cid,
		'name' => $wxname,
		'host' => "{$conf['httpshost']}:{$conf['httpsport']}"
	);

	$ret = httpPost($url, $data, true);
	if(empty($ret)) {
		conslog(__FUNCTION__, 1, "httpPost({$url}) empty.");
		return false;
	}
	
	$data = json_decode($ret, true);
	if(json_last_error() != JSON_ERROR_NONE) {
		conslog(__FUNCTION__, 1, "json_decode, data={$ret}");
		return false;
	}

	if(!isset($data['ret'])
	|| !isset($data['data'])) {
		conslog(__FUNCTION__, 1, "invdata, data={$ret}");
		return false;
	}

	if($data['ret']) {
		conslog(__FUNCTION__, 1, "code={$data['ret']}, msg={$data['msg']}");
		return false;
	}

	if(!is_array($data['data']))
		return true;

	$sync_acct_url = "https://weixin.panabit.com/panacloud/sync_acct.php";
	foreach($data['data'] as $action) {
		if($action['name'] == 'update_cid') {
			// if(_add_cloudconf(array('cid' => $action['cid']))) {
			if(file_put_contents("/etc/uuidgen.conf", $action['cid'])) {
				$cid = $action['cid'];
				conslog(__FUNCTION__, 0, "action={$action['name']} cid={$action['cid']}");
			}
			else
				conslog(__FUNCTION__, 1, "action={$action['name']} cid={$action['cid']}");
		}
		else
		if($action['name'] == 'sync_acct') {
			$now = time();
			$sql = "select concat(`user`, ':', `openid`) as `user` from cloud_platform.weixin_user where `enable` != 0 and `user` != `openid`";

			try {
				$sth = $nidb->prepare($sql);
				$sth->execute();
				$rows = $sth->fetchAll(\PDO::FETCH_COLUMN, 0);
			}
			catch (\NiDBException $e) {
				conslog(__FUNCTION__, 1, "action={$action['name']} DBException=" . $e->getMessage());
				return false;
			}

			if(!$rows || count($rows) <= 0)
				continue;

			$arg = array(
				'cid'  => $cid,
				'code' => $action['code'],
				'users'=> implode(' ', $rows),
			);

			$ret = httpPost($sync_acct_url, $arg, true);
			if(empty($ret)) {
				conslog(__FUNCTION__, 1, "httpPost({$sync_acct_url}) empty, action={$action['name']}");
				continue;
			}

			$jdata = json_decode($ret, true);
			if(json_last_error() != JSON_ERROR_NONE) {
				conslog(__FUNCTION__, 1, "json_decode, action={$action['name']} data={$ret}");
				continue;
			}

			if(!isset($jdata['ret'])) {
				conslog(__FUNCTION__, 1, "invdata, action={$action['name']} data={$ret}");
				continue;
			}
			
			if($jdata['ret']) 
				conslog(__FUNCTION__, 1, "action={$action['name']} code={$data['ret']}, msg={$jdata['msg']}");
			else
				conslog(__FUNCTION__, 0, "action={$action['name']} count={$jdata['data']}");
		}
		else
		if($action['name'] == 'sync_cmd') {
			$now = time();
			$path = LOGDATA_DIR . "/weixin/cmd";
			if(is_dir($path)
			&& $dh = opendir($path)) {
				while (($file = readdir($dh)) !== false) {
					if($file != "." && $file != "..") {
						if(is_dir($file))
							continue;

						/* Time out, 60s */
						$taskfile = "{$path}/{$file}";
						$ctime = filectime($taskfile);
						if ($ctime + 60 < $now) {
							unlink($taskfile);
							continue;
						}
					}
				}
				closedir($dh);
			}
			else 
				exec("mkdir -p {$path}");

			foreach($action['data'] as $val) {
				if(isset($val['task']) && $val['task'] != '') {
					$taskfile = "{$path}/{$val['task']}";
					if(file_exists($taskfile) == false) {
						file_put_contents($taskfile, json_encode($val));
						analysis_cmd($val);
					}
				}
			}
		}
	}

	return $cid;
}

function helpme()
{
	echo "err: seq args.\n";
	exit;
}

if($argc < 2) helpme();
$seq = intval($argv[1]);

$cid = _get_uuidgen();
if(($seq % 5) == 0) {
	$cid = checkguid($cid);
	if($cid == false)
		exit;
	cmd_netoff_send();
}

if(($seq % 3) == 0)
	cmd_netoff_result();

if($cid) {
	rebind($cid);
	unbind($cid);

	sendmsg_netoff($cid);
}
